'use strict';

const node_fs = require('node:fs');
const pathe = require('pathe');
const consola = require('consola');
const colorette = require('colorette');
const config = require('../shared/changelogen.12fd0d23.cjs');
require('node:os');
require('ofetch');
require('scule');
require('convert-gitmoji');
require('node-fetch-native');
require('node:path');
require('c12');
require('pkg-types');

async function githubMain(args) {
  const cwd = pathe.resolve(args.dir || "");
  process.chdir(cwd);
  const [subCommand, ..._versions] = args._;
  if (subCommand !== "release") {
    consola.log(
      "Usage: changelogen gh release [all|versions...] [--dir] [--token]"
    );
    process.exit(1);
  }
  const config$1 = await config.loadChangelogConfig(cwd, {});
  if (config$1.repo?.provider !== "github") {
    consola.error(
      "This command is only supported for github repository provider."
    );
    process.exit(1);
  }
  if (args.token) {
    config$1.tokens.github = args.token;
  }
  let changelogMd;
  if (typeof config$1.output === "string") {
    changelogMd = await node_fs.promises.readFile(pathe.resolve(config$1.output), "utf8").catch(() => null);
  }
  if (!changelogMd) {
    changelogMd = await config.getGithubChangelog(config$1).catch(() => null);
  }
  if (!changelogMd) {
    consola.error(`Cannot resolve CHANGELOG.md`);
    process.exit(1);
  }
  const changelogReleases = config.parseChangelogMarkdown(changelogMd).releases;
  let versions = [..._versions].map((v) => v.replace(/^v/, ""));
  if (versions[0] === "all") {
    versions = changelogReleases.map((r) => r.version).sort();
  } else if (versions.length === 0) {
    if (config$1.newVersion) {
      versions = [config$1.newVersion];
    } else if (changelogReleases.length > 0) {
      versions = [changelogReleases[0].version];
    }
  }
  if (versions.length === 0) {
    consola.error(`No versions specified to release!`);
    process.exit(1);
  }
  for (const version of versions) {
    const release = changelogReleases.find((r) => r.version === version);
    if (!release) {
      consola.warn(
        `No matching changelog entry found for ${version} in CHANGELOG.md. Skipping!`
      );
      continue;
    }
    if (!release.body || !release.version) {
      consola.warn(
        `Changelog entry for ${version} in CHANGELOG.md is missing body or version. Skipping!`
      );
      continue;
    }
    await githubRelease(config$1, {
      version: release.version,
      body: release.body
    });
  }
}
async function githubRelease(config$1, release) {
  if (!config$1.tokens.github) {
    config$1.tokens.github = await config.resolveGithubToken(config$1).catch(
      () => void 0
    );
  }
  const result = await config.syncGithubRelease(config$1, release);
  if (result.status === "manual") {
    if (result.error) {
      consola.error(result.error);
      process.exitCode = 1;
    }
    const open = await import('open').then((r) => r.default);
    await open(result.url).then(() => {
      consola.info(`Followup in the browser to manually create the release.`);
    }).catch(() => {
      consola.info(
        `Open this link to manually create a release: 
` + colorette.underline(colorette.cyan(result.url)) + "\n"
      );
    });
  } else {
    consola.success(
      `Synced ${colorette.cyan(`v${release.version}`)} to Github releases!`
    );
  }
}

exports.default = githubMain;
exports.githubRelease = githubRelease;
